@@ -426,7 +426,7 @@ static PHP_METHOD(MongoDB_Driver_ClientEncryption, rewrapManyDataKey)
 	}
 
 	if (options && php_array_existsc(options, "masterKey")) {
-		zval* zmasterkey = php_array_fetchc(options, "masterKey");
+		zval* zmasterkey = php_array_fetchc_deref(options, "masterKey");
 
 		if (Z_TYPE_P(zmasterkey) != IS_OBJECT && Z_TYPE_P(zmasterkey) != IS_ARRAY) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"masterKey\" option to be array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zmasterkey));
@@ -550,7 +550,7 @@ static mongoc_client_encryption_opts_t* phongo_clientencryption_opts_from_zval(z
 	}
 
 	if (php_array_existsc(options, "keyVaultClient")) {
-		zval* key_vault_client = php_array_fetch(options, "keyVaultClient");
+		zval* key_vault_client = php_array_fetchc_deref(options, "keyVaultClient");
 
 		if (Z_TYPE_P(key_vault_client) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(key_vault_client), php_phongo_manager_ce)) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultClient\" option to be %s, %s given", ZSTR_VAL(php_phongo_manager_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(key_vault_client));
@@ -598,7 +598,7 @@ static mongoc_client_encryption_opts_t* phongo_clientencryption_opts_from_zval(z
 	}
 
 	if (php_array_existsc(options, "kmsProviders")) {
-		zval*  kms_providers  = php_array_fetchc(options, "kmsProviders");
+		zval*  kms_providers  = php_array_fetchc_deref(options, "kmsProviders");
 		bson_t bson_providers = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(kms_providers) != IS_ARRAY && Z_TYPE_P(kms_providers) != IS_OBJECT) {
@@ -616,7 +616,7 @@ static mongoc_client_encryption_opts_t* phongo_clientencryption_opts_from_zval(z
 	}
 
 	if (php_array_existsc(options, "tlsOptions")) {
-		zval*  tls_options  = php_array_fetchc(options, "tlsOptions");
+		zval*  tls_options  = php_array_fetchc_deref(options, "tlsOptions");
 		bson_t bson_options = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(tls_options) != IS_ARRAY && Z_TYPE_P(tls_options) != IS_OBJECT) {
@@ -707,7 +707,7 @@ static mongoc_client_encryption_datakey_opts_t* phongo_clientencryption_datakey_
 	}
 
 	if (php_array_existsc(options, "keyAltNames")) {
-		zval*      zkeyaltnames = php_array_fetchc(options, "keyAltNames");
+		zval*      zkeyaltnames = php_array_fetchc_deref(options, "keyAltNames");
 		HashTable* ht_data;
 		uint32_t   keyaltnames_count;
 		char**     keyaltnames;
@@ -769,7 +769,7 @@ static mongoc_client_encryption_datakey_opts_t* phongo_clientencryption_datakey_
 	}
 
 	if (php_array_existsc(options, "keyMaterial")) {
-		zval* keyMaterial = php_array_fetchc(options, "keyMaterial");
+		zval* keyMaterial = php_array_fetchc_deref(options, "keyMaterial");
 
 		if (Z_TYPE_P(keyMaterial) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(keyMaterial), php_phongo_binary_ce)) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyMaterial\" option to be %s, %s given", ZSTR_VAL(php_phongo_binary_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(keyMaterial));
@@ -780,7 +780,7 @@ static mongoc_client_encryption_datakey_opts_t* phongo_clientencryption_datakey_
 	}
 
 	if (php_array_existsc(options, "masterKey")) {
-		zval*  zmasterkey = php_array_fetchc(options, "masterKey");
+		zval*  zmasterkey = php_array_fetchc_deref(options, "masterKey");
 		bson_t masterkey  = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(zmasterkey) != IS_OBJECT && Z_TYPE_P(zmasterkey) != IS_ARRAY) {
@@ -850,6 +850,17 @@ static mongoc_client_encryption_encrypt_range_opts_t* phongo_clientencryption_en
 		return opts;
 	}
 
+	if (php_array_existsc(options, "trimFactor")) {
+		int64_t trimfactor = php_array_fetchc_long(options, "trimFactor");
+
+		if (trimfactor < 0 || trimfactor > INT32_MAX) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"trimFactor\" range option to be a positive 32-bit integer, %" PRId64 " given", trimfactor);
+			goto cleanup;
+		}
+
+		mongoc_client_encryption_encrypt_range_opts_set_trim_factor(opts, (int32_t) trimfactor);
+	}
+
 	if (php_array_existsc(options, "sparsity")) {
 		int64_t sparsity = php_array_fetchc_long(options, "sparsity");
 
@@ -921,7 +932,7 @@ static mongoc_client_encryption_encrypt_opts_t* phongo_clientencryption_encrypt_
 	}
 
 	if (php_array_existsc(options, "contentionFactor")) {
-		mongoc_client_encryption_encrypt_opts_set_contention_factor(opts, php_array_fetch_long(options, "contentionFactor"));
+		mongoc_client_encryption_encrypt_opts_set_contention_factor(opts, php_array_fetchc_long(options, "contentionFactor"));
 	}
 
 	if (php_array_existsc(options, "keyId")) {
@@ -943,7 +954,7 @@ static mongoc_client_encryption_encrypt_opts_t* phongo_clientencryption_encrypt_
 		int       plen;
 		zend_bool pfree;
 
-		keyaltname = php_array_fetch_string(options, "keyAltName", &plen, &pfree);
+		keyaltname = php_array_fetchc_string(options, "keyAltName", &plen, &pfree);
 		mongoc_client_encryption_encrypt_opts_set_keyaltname(opts, keyaltname);
 
 		if (pfree) {
@@ -956,7 +967,7 @@ static mongoc_client_encryption_encrypt_opts_t* phongo_clientencryption_encrypt_
 		int       plen;
 		zend_bool pfree;
 
-		algorithm = php_array_fetch_string(options, "algorithm", &plen, &pfree);
+		algorithm = php_array_fetchc_string(options, "algorithm", &plen, &pfree);
 		mongoc_client_encryption_encrypt_opts_set_algorithm(opts, algorithm);
 
 		if (pfree) {
@@ -969,7 +980,7 @@ static mongoc_client_encryption_encrypt_opts_t* phongo_clientencryption_encrypt_
 		int       plen;
 		zend_bool pfree;
 
-		querytype = php_array_fetch_string(options, "queryType", &plen, &pfree);
+		querytype = php_array_fetchc_string(options, "queryType", &plen, &pfree);
 		mongoc_client_encryption_encrypt_opts_set_query_type(opts, querytype);
 
 		if (pfree) {
@@ -980,7 +991,7 @@ static mongoc_client_encryption_encrypt_opts_t* phongo_clientencryption_encrypt_
 	if (php_array_existsc(options, "rangeOpts")) {
 		mongoc_client_encryption_encrypt_range_opts_t* range_opts;
 
-		range_opts = phongo_clientencryption_encrypt_range_opts_from_zval(php_array_fetchc(options, "rangeOpts"));
+		range_opts = phongo_clientencryption_encrypt_range_opts_from_zval(php_array_fetchc_deref(options, "rangeOpts"));
 
 		if (!range_opts) {
 			/* Exception already thrown */
