@@ -143,12 +143,22 @@ static PHP_METHOD(MongoDB_Driver_Cursor, toArray)
 static PHP_METHOD(MongoDB_Driver_Cursor, getId)
 {
 	php_phongo_cursor_t* intern;
+	zend_bool            asInt64 = false;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
-	PHONGO_PARSE_PARAMETERS_NONE();
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_BOOL(asInt64)
+	PHONGO_PARSE_PARAMETERS_END();
 
-	php_phongo_cursor_id_new_from_id(return_value, mongoc_cursor_get_id(intern->cursor));
+	if (asInt64) {
+		phongo_int64_new(return_value, mongoc_cursor_get_id(intern->cursor));
+	} else {
+		php_error_docref(NULL, E_DEPRECATED, "The method \"MongoDB\\Driver\\Cursor::getId\" will no longer return a \"MongoDB\\Driver\\CursorId\" instance in the future. Pass \"true\" as argument to change to the new behavior and receive a \"MongoDB\\BSON\\Int64\" instance instead.");
+
+		php_phongo_cursor_id_new_from_id(return_value, mongoc_cursor_get_id(intern->cursor));
+	}
 }
 
 /* Returns the Server object to which this cursor is attached */
@@ -446,7 +456,7 @@ static void phongo_cursor_init(zval* return_value, zval* manager, mongoc_cursor_
 
 	intern            = Z_CURSOR_OBJ_P(return_value);
 	intern->cursor    = cursor;
-	intern->server_id = mongoc_cursor_get_hint(cursor);
+	intern->server_id = mongoc_cursor_get_server_id(cursor);
 	intern->advanced  = false;
 	intern->current   = 0;
 
@@ -485,7 +495,7 @@ bool phongo_cursor_init_for_query(zval* return_value, zval* manager, mongoc_curs
 	php_phongo_cursor_t* intern;
 
 	/* Advancing the cursor before phongo_cursor_init ensures that a server
-	 * stream is obtained before mongoc_cursor_get_hint() is called. */
+	 * stream is obtained before mongoc_cursor_get_server_id() is called. */
 	if (!phongo_cursor_advance_and_check_for_error(cursor)) {
 		/* Exception should already have been thrown */
 		return false;
