@@ -31,7 +31,6 @@ zend_class_entry* php_phongo_commandsucceededevent_ce;
 
 PHONGO_DISABLED_CONSTRUCTOR(MongoDB_Driver_Monitoring_CommandSucceededEvent)
 
-/* Returns the command name for this event */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getCommandName)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -43,7 +42,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getCommandNam
 	RETVAL_STRING(intern->command_name);
 }
 
-/* Returns the database name for this event */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getDatabaseName)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -55,7 +53,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getDatabaseNa
 	RETVAL_STRING(intern->database_name);
 }
 
-/* Returns the event's duration in microseconds */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getDurationMicros)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -67,7 +64,15 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getDurationMi
 	RETURN_LONG(intern->duration_micros);
 }
 
-/* Returns the event's operation ID */
+static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getHost)
+{
+	php_phongo_commandsucceededevent_t* intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETVAL_STRING(intern->host.host);
+}
+
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getOperationId)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -81,7 +86,15 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getOperationI
 	RETVAL_STRING(operation_id);
 }
 
-/* Returns the reply document associated with the event */
+static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getPort)
+{
+	php_phongo_commandsucceededevent_t* intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETVAL_LONG(intern->host.port);
+}
+
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getReply)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -101,7 +114,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getReply)
 	RETURN_ZVAL(&state.zchild, 0, 1);
 }
 
-/* Returns the event's request ID */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getRequestId)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -115,7 +127,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getRequestId)
 	RETVAL_STRING(request_id);
 }
 
-/* Returns the Server from which the event originated */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServer)
 {
 	php_phongo_commandsucceededevent_t* intern;
@@ -127,7 +138,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServer)
 	phongo_server_init(return_value, &intern->manager, intern->server_id);
 }
 
-/* Returns the event's service ID */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServiceId)
 {
 	php_phongo_commandsucceededevent_t* intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
@@ -141,7 +151,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServiceId)
 	phongo_objectid_new(return_value, &intern->service_id);
 }
 
-/* Returns the event's server connection ID */
 static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServerConnectionId)
 {
 	php_phongo_commandsucceededevent_t* intern = Z_COMMANDSUCCEEDEDEVENT_OBJ_P(getThis());
@@ -162,12 +171,6 @@ static PHP_METHOD(MongoDB_Driver_Monitoring_CommandSucceededEvent, getServerConn
 	RETURN_LONG(intern->server_connection_id);
 }
 
-/**
- * Event thrown when a command has succeeded to execute.
- *
- * This class is only constructed internally.
- */
-
 /* MongoDB\Driver\Monitoring\CommandSucceededEvent object handlers */
 static zend_object_handlers php_phongo_handler_commandsucceededevent;
 
@@ -217,14 +220,13 @@ static HashTable* php_phongo_commandsucceededevent_get_debug_info(phongo_compat_
 
 	intern   = Z_OBJ_COMMANDSUCCEEDEDEVENT(PHONGO_COMPAT_GET_OBJ(object));
 	*is_temp = 1;
-	array_init_size(&retval, 6);
+	array_init_size(&retval, 10);
 
+	ADD_ASSOC_STRING(&retval, "host", intern->host.host);
+	ADD_ASSOC_LONG_EX(&retval, "port", intern->host.port);
 	ADD_ASSOC_STRING(&retval, "commandName", intern->command_name);
 	ADD_ASSOC_INT64(&retval, "durationMicros", intern->duration_micros);
 
-	snprintf(operation_id, sizeof(operation_id), "%" PRId64, intern->operation_id);
-	ADD_ASSOC_STRING(&retval, "operationId", operation_id);
-
 	if (!php_phongo_bson_to_zval_ex(intern->reply, &reply_state)) {
 		zval_ptr_dtor(&reply_state.zchild);
 		goto done;
@@ -232,6 +234,9 @@ static HashTable* php_phongo_commandsucceededevent_get_debug_info(phongo_compat_
 
 	ADD_ASSOC_ZVAL(&retval, "reply", &reply_state.zchild);
 
+	snprintf(operation_id, sizeof(operation_id), "%" PRId64, intern->operation_id);
+	ADD_ASSOC_STRING(&retval, "operationId", operation_id);
+
 	snprintf(request_id, sizeof(request_id), "%" PRId64, intern->request_id);
 	ADD_ASSOC_STRING(&retval, "requestId", request_id);
 
