@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,26 +42,6 @@
 #define BSON_OS 1
 
 
-/*
- * Define to 1 if we have access to GCC 32-bit atomic builtins.
- * While this requires GCC 4.1+ in most cases, it is also architecture
- * dependent. For example, some PPC or ARM systems may not have it even
- * if it is a recent GCC version.
- */
-#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
-#if BSON_HAVE_ATOMIC_32_ADD_AND_FETCH != 1
-# undef BSON_HAVE_ATOMIC_32_ADD_AND_FETCH
-#endif
-
-/*
- * Similarly, define to 1 if we have access to GCC 64-bit atomic builtins.
- */
-#define BSON_HAVE_ATOMIC_64_ADD_AND_FETCH 1
-#if BSON_HAVE_ATOMIC_64_ADD_AND_FETCH != 1
-# undef BSON_HAVE_ATOMIC_64_ADD_AND_FETCH
-#endif
-
-
 /*
  * Define to 1 if you have clock_gettime() available.
  */
@@ -107,15 +87,6 @@
 #endif
 
 
-/*
- * Define to 1 if you have reallocf available on your platform.
- */
-#define BSON_HAVE_REALLOCF 1
-#if BSON_HAVE_REALLOCF != 1
-# undef BSON_HAVE_REALLOCF
-#endif
-
-
 /*
  * Define to 1 if you have struct timespec available on your platform.
  */
@@ -135,17 +106,20 @@
 
 
 /*
- * Define to 1 if you have SYS_gettid syscall
+ * Define to 1 if you have rand_r available on your platform.
  */
-#define BSON_HAVE_SYSCALL_TID 0
-#if BSON_HAVE_SYSCALL_TID != 1
-# undef BSON_HAVE_SYSCALL_TID
-#endif
-
 #define BSON_HAVE_RAND_R 1
 #if BSON_HAVE_RAND_R != 1
 # undef BSON_HAVE_RAND_R
 #endif
 
 
+/*
+ * Define to 1 if you have strlcpy available on your platform.
+ */
+#define BSON_HAVE_STRLCPY 0
+#if BSON_HAVE_STRLCPY != 1
+# undef BSON_HAVE_STRLCPY
+#endif
+
 #endif /* BSON_CONFIG_H */
