@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,23 +14,23 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_COLLECTION_H
 #define MONGOC_COLLECTION_H
 
 #include <bson/bson.h>
 
-#include "mongoc/mongoc-change-stream.h"
-#include "mongoc/mongoc-macros.h"
-#include "mongoc/mongoc-bulk-operation.h"
-#include "mongoc/mongoc-flags.h"
-#include "mongoc/mongoc-cursor.h"
-#include "mongoc/mongoc-index.h"
-#include "mongoc/mongoc-read-prefs.h"
-#include "mongoc/mongoc-read-concern.h"
-#include "mongoc/mongoc-write-concern.h"
-#include "mongoc/mongoc-find-and-modify.h"
+#include "mongoc-change-stream.h"
+#include "mongoc-macros.h"
+#include "mongoc-bulk-operation.h"
+#include "mongoc-flags.h"
+#include "mongoc-cursor.h"
+#include "mongoc-index.h"
+#include "mongoc-read-prefs.h"
+#include "mongoc-read-concern.h"
+#include "mongoc-write-concern.h"
+#include "mongoc-find-and-modify.h"
 
 BSON_BEGIN_DECLS
 
@@ -42,12 +42,11 @@ mongoc_collection_aggregate (mongoc_collection_t *collection,
                              mongoc_query_flags_t flags,
                              const bson_t *pipeline,
                              const bson_t *opts,
-                             const mongoc_read_prefs_t *read_prefs)
-   BSON_GNUC_WARN_UNUSED_RESULT;
+                             const mongoc_read_prefs_t *read_prefs) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (void)
 mongoc_collection_destroy (mongoc_collection_t *collection);
 MONGOC_EXPORT (mongoc_collection_t *)
-mongoc_collection_copy (mongoc_collection_t *collection);
+mongoc_collection_copy (mongoc_collection_t *collection) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (mongoc_cursor_t *)
 mongoc_collection_command (mongoc_collection_t *collection,
                            mongoc_query_flags_t flags,
@@ -56,8 +55,7 @@ mongoc_collection_command (mongoc_collection_t *collection,
                            uint32_t batch_size,
                            const bson_t *command,
                            const bson_t *fields,
-                           const mongoc_read_prefs_t *read_prefs)
-   BSON_GNUC_WARN_UNUSED_RESULT;
+                           const mongoc_read_prefs_t *read_prefs) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (bool)
 mongoc_collection_read_command_with_opts (mongoc_collection_t *collection,
                                           const bson_t *command,
@@ -66,19 +64,15 @@ mongoc_collection_read_command_with_opts (mongoc_collection_t *collection,
                                           bson_t *reply,
                                           bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_write_command_with_opts (mongoc_collection_t *collection,
-                                           const bson_t *command,
-                                           const bson_t *opts,
-                                           bson_t *reply,
-                                           bson_error_t *error);
+mongoc_collection_write_command_with_opts (
+   mongoc_collection_t *collection, const bson_t *command, const bson_t *opts, bson_t *reply, bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_read_write_command_with_opts (
-   mongoc_collection_t *collection,
-   const bson_t *command,
-   const mongoc_read_prefs_t *read_prefs /* IGNORED */,
-   const bson_t *opts,
-   bson_t *reply,
-   bson_error_t *error);
+mongoc_collection_read_write_command_with_opts (mongoc_collection_t *collection,
+                                                const bson_t *command,
+                                                const mongoc_read_prefs_t *read_prefs /* IGNORED */,
+                                                const bson_t *opts,
+                                                bson_t *reply,
+                                                bson_error_t *error);
 MONGOC_EXPORT (bool)
 mongoc_collection_command_with_opts (mongoc_collection_t *collection,
                                      const bson_t *command,
@@ -100,8 +94,7 @@ mongoc_collection_count (mongoc_collection_t *collection,
                          int64_t limit,
                          const mongoc_read_prefs_t *read_prefs,
                          bson_error_t *error)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_count_documents or
-                             mongoc_collection_estimated_document_count);
+   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_count_documents or mongoc_collection_estimated_document_count);
 MONGOC_EXPORT (int64_t)
 mongoc_collection_count_with_opts (mongoc_collection_t *collection,
                                    mongoc_query_flags_t flags,
@@ -111,18 +104,13 @@ mongoc_collection_count_with_opts (mongoc_collection_t *collection,
                                    const bson_t *opts,
                                    const mongoc_read_prefs_t *read_prefs,
                                    bson_error_t *error)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_count_documents or
-                             mongoc_collection_estimated_document_count);
+   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_count_documents or mongoc_collection_estimated_document_count);
 MONGOC_EXPORT (bool)
 mongoc_collection_drop (mongoc_collection_t *collection, bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_drop_with_opts (mongoc_collection_t *collection,
-                                  const bson_t *opts,
-                                  bson_error_t *error);
+mongoc_collection_drop_with_opts (mongoc_collection_t *collection, const bson_t *opts, bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_drop_index (mongoc_collection_t *collection,
-                              const char *index_name,
-                              bson_error_t *error);
+mongoc_collection_drop_index (mongoc_collection_t *collection, const char *index_name, bson_error_t *error);
 MONGOC_EXPORT (bool)
 mongoc_collection_drop_index_with_opts (mongoc_collection_t *collection,
                                         const char *index_name,
@@ -139,20 +127,35 @@ mongoc_collection_create_index_with_opts (mongoc_collection_t *collection,
                                           const mongoc_index_opt_t *opt,
                                           const bson_t *opts,
                                           bson_t *reply,
-                                          bson_error_t *error)
-   BSON_GNUC_DEPRECATED;
+                                          bson_error_t *error) BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (bool)
 mongoc_collection_ensure_index (mongoc_collection_t *collection,
                                 const bson_t *keys,
                                 const mongoc_index_opt_t *opt,
                                 bson_error_t *error) BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (mongoc_cursor_t *)
-mongoc_collection_find_indexes (mongoc_collection_t *collection,
-                                bson_error_t *error)
+mongoc_collection_find_indexes (mongoc_collection_t *collection, bson_error_t *error) BSON_GNUC_WARN_UNUSED_RESULT
    BSON_GNUC_DEPRECATED_FOR (mongoc_collection_find_indexes_with_opts);
 MONGOC_EXPORT (mongoc_cursor_t *)
 mongoc_collection_find_indexes_with_opts (mongoc_collection_t *collection,
-                                          const bson_t *opts);
+                                          const bson_t *opts) BSON_GNUC_WARN_UNUSED_RESULT;
+
+typedef struct _mongoc_index_model_t mongoc_index_model_t;
+
+MONGOC_EXPORT (mongoc_index_model_t *)
+mongoc_index_model_new (const bson_t *keys, const bson_t *opts);
+
+MONGOC_EXPORT (void) mongoc_index_model_destroy (mongoc_index_model_t *model);
+
+MONGOC_EXPORT (bool)
+mongoc_collection_create_indexes_with_opts (mongoc_collection_t *collection,
+                                            mongoc_index_model_t *const *models,
+                                            size_t n_models,
+                                            const bson_t *opts,
+                                            bson_t *reply,
+                                            bson_error_t *error);
+
+
 MONGOC_EXPORT (mongoc_cursor_t *)
 mongoc_collection_find (mongoc_collection_t *collection,
                         mongoc_query_flags_t flags,
@@ -162,14 +165,12 @@ mongoc_collection_find (mongoc_collection_t *collection,
                         const bson_t *query,
                         const bson_t *fields,
                         const mongoc_read_prefs_t *read_prefs)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_find_with_opts)
-      BSON_GNUC_WARN_UNUSED_RESULT;
+   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_find_with_opts) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (mongoc_cursor_t *)
 mongoc_collection_find_with_opts (mongoc_collection_t *collection,
                                   const bson_t *filter,
                                   const bson_t *opts,
-                                  const mongoc_read_prefs_t *read_prefs)
-   BSON_GNUC_WARN_UNUSED_RESULT;
+                                  const mongoc_read_prefs_t *read_prefs) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (bool)
 mongoc_collection_insert (mongoc_collection_t *collection,
                           mongoc_insert_flags_t flags,
@@ -177,11 +178,8 @@ mongoc_collection_insert (mongoc_collection_t *collection,
                           const mongoc_write_concern_t *write_concern,
                           bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_insert_one (mongoc_collection_t *collection,
-                              const bson_t *document,
-                              const bson_t *opts,
-                              bson_t *reply,
-                              bson_error_t *error);
+mongoc_collection_insert_one (
+   mongoc_collection_t *collection, const bson_t *document, const bson_t *opts, bson_t *reply, bson_error_t *error);
 MONGOC_EXPORT (bool)
 mongoc_collection_insert_many (mongoc_collection_t *collection,
                                const bson_t **documents,
@@ -195,8 +193,7 @@ mongoc_collection_insert_bulk (mongoc_collection_t *collection,
                                const bson_t **documents,
                                uint32_t n_documents,
                                const mongoc_write_concern_t *write_concern,
-                               bson_error_t *error)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_insert_many);
+                               bson_error_t *error) BSON_GNUC_DEPRECATED_FOR (mongoc_collection_insert_many);
 MONGOC_EXPORT (bool)
 mongoc_collection_update (mongoc_collection_t *collection,
                           mongoc_update_flags_t flags,
@@ -231,15 +228,13 @@ mongoc_collection_delete (mongoc_collection_t *collection,
                           const bson_t *selector,
                           const mongoc_write_concern_t *write_concern,
                           bson_error_t *error)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_delete_one or
-                             mongoc_collection_delete_many);
+   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_delete_one or mongoc_collection_delete_many);
 MONGOC_EXPORT (bool)
 mongoc_collection_save (mongoc_collection_t *collection,
                         const bson_t *document,
                         const mongoc_write_concern_t *write_concern,
                         bson_error_t *error)
-   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_insert_one or
-                             mongoc_collection_replace_one);
+   BSON_GNUC_DEPRECATED_FOR (mongoc_collection_insert_one or mongoc_collection_replace_one);
 MONGOC_EXPORT (bool)
 mongoc_collection_remove (mongoc_collection_t *collection,
                           mongoc_remove_flags_t flags,
@@ -247,17 +242,11 @@ mongoc_collection_remove (mongoc_collection_t *collection,
                           const mongoc_write_concern_t *write_concern,
                           bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_delete_one (mongoc_collection_t *collection,
-                              const bson_t *selector,
-                              const bson_t *opts,
-                              bson_t *reply,
-                              bson_error_t *error);
+mongoc_collection_delete_one (
+   mongoc_collection_t *collection, const bson_t *selector, const bson_t *opts, bson_t *reply, bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_delete_many (mongoc_collection_t *collection,
-                               const bson_t *selector,
-                               const bson_t *opts,
-                               bson_t *reply,
-                               bson_error_t *error);
+mongoc_collection_delete_many (
+   mongoc_collection_t *collection, const bson_t *selector, const bson_t *opts, bson_t *reply, bson_error_t *error);
 MONGOC_EXPORT (bool)
 mongoc_collection_rename (mongoc_collection_t *collection,
                           const char *new_db,
@@ -272,12 +261,11 @@ mongoc_collection_rename_with_opts (mongoc_collection_t *collection,
                                     const bson_t *opts,
                                     bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_find_and_modify_with_opts (
-   mongoc_collection_t *collection,
-   const bson_t *query,
-   const mongoc_find_and_modify_opts_t *opts,
-   bson_t *reply,
-   bson_error_t *error);
+mongoc_collection_find_and_modify_with_opts (mongoc_collection_t *collection,
+                                             const bson_t *query,
+                                             const mongoc_find_and_modify_opts_t *opts,
+                                             bson_t *reply,
+                                             bson_error_t *error);
 MONGOC_EXPORT (bool)
 mongoc_collection_find_and_modify (mongoc_collection_t *collection,
                                    const bson_t *query,
@@ -290,52 +278,41 @@ mongoc_collection_find_and_modify (mongoc_collection_t *collection,
                                    bson_t *reply,
                                    bson_error_t *error);
 MONGOC_EXPORT (bool)
-mongoc_collection_stats (mongoc_collection_t *collection,
-                         const bson_t *options,
-                         bson_t *reply,
-                         bson_error_t *error) BSON_GNUC_DEPRECATED;
+mongoc_collection_stats (mongoc_collection_t *collection, const bson_t *options, bson_t *reply, bson_error_t *error)
+   BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (mongoc_bulk_operation_t *)
-mongoc_collection_create_bulk_operation (
-   mongoc_collection_t *collection,
-   bool ordered,
-   const mongoc_write_concern_t *write_concern) BSON_GNUC_WARN_UNUSED_RESULT
+mongoc_collection_create_bulk_operation (mongoc_collection_t *collection,
+                                         bool ordered,
+                                         const mongoc_write_concern_t *write_concern) BSON_GNUC_WARN_UNUSED_RESULT
    BSON_GNUC_DEPRECATED_FOR (mongoc_collection_create_bulk_operation_with_opts);
 MONGOC_EXPORT (mongoc_bulk_operation_t *)
-mongoc_collection_create_bulk_operation_with_opts (
-   mongoc_collection_t *collection,
-   const bson_t *opts) BSON_GNUC_WARN_UNUSED_RESULT;
+mongoc_collection_create_bulk_operation_with_opts (mongoc_collection_t *collection,
+                                                   const bson_t *opts) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (const mongoc_read_prefs_t *)
 mongoc_collection_get_read_prefs (const mongoc_collection_t *collection);
 MONGOC_EXPORT (void)
-mongoc_collection_set_read_prefs (mongoc_collection_t *collection,
-                                  const mongoc_read_prefs_t *read_prefs);
+mongoc_collection_set_read_prefs (mongoc_collection_t *collection, const mongoc_read_prefs_t *read_prefs);
 MONGOC_EXPORT (const mongoc_read_concern_t *)
 mongoc_collection_get_read_concern (const mongoc_collection_t *collection);
 MONGOC_EXPORT (void)
-mongoc_collection_set_read_concern (mongoc_collection_t *collection,
-                                    const mongoc_read_concern_t *read_concern);
+mongoc_collection_set_read_concern (mongoc_collection_t *collection, const mongoc_read_concern_t *read_concern);
 MONGOC_EXPORT (const mongoc_write_concern_t *)
 mongoc_collection_get_write_concern (const mongoc_collection_t *collection);
 MONGOC_EXPORT (void)
-mongoc_collection_set_write_concern (
-   mongoc_collection_t *collection,
-   const mongoc_write_concern_t *write_concern);
+mongoc_collection_set_write_concern (mongoc_collection_t *collection, const mongoc_write_concern_t *write_concern);
 MONGOC_EXPORT (const char *)
 mongoc_collection_get_name (mongoc_collection_t *collection);
 MONGOC_EXPORT (const bson_t *)
-mongoc_collection_get_last_error (const mongoc_collection_t *collection)
-   BSON_GNUC_DEPRECATED;
+mongoc_collection_get_last_error (const mongoc_collection_t *collection) BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (char *)
-mongoc_collection_keys_to_index_string (const bson_t *keys);
+mongoc_collection_keys_to_index_string (const bson_t *keys) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (bool)
-mongoc_collection_validate (mongoc_collection_t *collection,
-                            const bson_t *options,
-                            bson_t *reply,
-                            bson_error_t *error) BSON_GNUC_DEPRECATED;
+mongoc_collection_validate (mongoc_collection_t *collection, const bson_t *options, bson_t *reply, bson_error_t *error)
+   BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (mongoc_change_stream_t *)
 mongoc_collection_watch (const mongoc_collection_t *coll,
                          const bson_t *pipeline,
-                         const bson_t *opts);
+                         const bson_t *opts) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (int64_t)
 mongoc_collection_count_documents (mongoc_collection_t *coll,
                                    const bson_t *filter,
@@ -344,12 +321,11 @@ mongoc_collection_count_documents (mongoc_collection_t *coll,
                                    bson_t *reply,
                                    bson_error_t *error);
 MONGOC_EXPORT (int64_t)
-mongoc_collection_estimated_document_count (
-   mongoc_collection_t *coll,
-   const bson_t *opts,
-   const mongoc_read_prefs_t *read_prefs,
-   bson_t *reply,
-   bson_error_t *error);
+mongoc_collection_estimated_document_count (mongoc_collection_t *coll,
+                                            const bson_t *opts,
+                                            const mongoc_read_prefs_t *read_prefs,
+                                            bson_t *reply,
+                                            bson_error_t *error);
 
 BSON_END_DECLS
 
