@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -137,6 +137,9 @@ mongoc_compressor_name_to_id (const char *compressor)
    return -1;
 }
 
+// To support unchecked casts from `unsigned long` to `size_t`.
+BSON_STATIC_ASSERT2 (size_t_gte_ulong, SIZE_MAX >= ULONG_MAX);
+
 bool
 mongoc_uncompress (int32_t compressor_id,
                    const uint8_t *compressed,
@@ -144,13 +147,17 @@ mongoc_uncompress (int32_t compressor_id,
                    uint8_t *uncompressed,
                    size_t *uncompressed_len)
 {
+   BSON_ASSERT_PARAM (compressed);
+   BSON_ASSERT_PARAM (uncompressed);
+   BSON_ASSERT_PARAM (uncompressed_len);
+
    TRACE ("Uncompressing with '%s' (%d)", mongoc_compressor_id_to_name (compressor_id), compressor_id);
 
    switch (compressor_id) {
    case MONGOC_COMPRESSOR_SNAPPY_ID: {
 #ifdef MONGOC_ENABLE_COMPRESSION_SNAPPY
-      snappy_status status;
-      status = snappy_uncompress ((const char *) compressed, compressed_len, (char *) uncompressed, uncompressed_len);
+      const snappy_status status =
+         snappy_uncompress ((const char *) compressed, compressed_len, (char *) uncompressed, uncompressed_len);
 
       return status == SNAPPY_OK;
 #else
@@ -162,11 +169,28 @@ mongoc_uncompress (int32_t compressor_id,
 
    case MONGOC_COMPRESSOR_ZLIB_ID: {
 #ifdef MONGOC_ENABLE_COMPRESSION_ZLIB
-      BSON_ASSERT (bson_in_range_unsigned (unsigned_long, compressed_len));
-      const int ok =
-         uncompress (uncompressed, (unsigned long *) uncompressed_len, compressed, (unsigned long) compressed_len);
+      // Malformed message: unrepresentable.
+      if (BSON_UNLIKELY (!bson_in_range_unsigned (unsigned_long, compressed_len))) {
+         return false;
+      }
 
-      return ok == Z_OK;
+      // Malformed message: unrepresentable.
+      if (BSON_UNLIKELY (!bson_in_range_unsigned (unsigned_long, *uncompressed_len))) {
+         return false;
+      }
+
+      uLong actual_uncompressed_len = (uLong) *uncompressed_len;
+
+      const int res =
+         uncompress (uncompressed, &actual_uncompressed_len, (const Bytef *) compressed, (uLong) compressed_len);
+
+      if (BSON_UNLIKELY (res != Z_OK)) {
+         return false;
+      }
+
+      *uncompressed_len = (size_t) actual_uncompressed_len;
+
+      return true;
 #else
       MONGOC_WARNING ("Received zlib compressed opcode, but zlib "
                       "compression is not compiled in");
@@ -176,15 +200,15 @@ mongoc_uncompress (int32_t compressor_id,
 
    case MONGOC_COMPRESSOR_ZSTD_ID: {
 #ifdef MONGOC_ENABLE_COMPRESSION_ZSTD
-      int ok;
+      const size_t res = ZSTD_decompress (uncompressed, *uncompressed_len, compressed, compressed_len);
 
-      ok = ZSTD_decompress ((void *) uncompressed, *uncompressed_len, (const void *) compressed, compressed_len);
-
-      if (!ZSTD_isError (ok)) {
-         *uncompressed_len = ok;
+      if (BSON_UNLIKELY (ZSTD_isError (res))) {
+         return false;
       }
 
-      return !ZSTD_isError (ok);
+      *uncompressed_len = res;
+
+      return true;
 #else
       MONGOC_WARNING ("Received zstd compressed opcode, but zstd "
                       "compression is not compiled in");
@@ -192,6 +216,10 @@ mongoc_uncompress (int32_t compressor_id,
 #endif
    }
    case MONGOC_COMPRESSOR_NOOP_ID:
+      // Malformed message: not enough space.
+      if (BSON_UNLIKELY (*uncompressed_len < compressed_len)) {
+         return false;
+      }
       memcpy (uncompressed, compressed, compressed_len);
       *uncompressed_len = compressed_len;
       return true;
