@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,12 +18,12 @@
 #include <errno.h>
 #include <string.h>
 
-#include "mongoc/mongoc-counters-private.h"
-#include "mongoc/mongoc-errno-private.h"
-#include "mongoc/mongoc-socket-private.h"
-#include "mongoc/mongoc-host-list.h"
-#include "mongoc/mongoc-socket-private.h"
-#include "mongoc/mongoc-trace-private.h"
+#include "mongoc-counters-private.h"
+#include "mongoc-errno-private.h"
+#include "mongoc-socket-private.h"
+#include "mongoc-host-list.h"
+#include "mongoc-socket-private.h"
+#include "mongoc-trace-private.h"
 #ifdef _WIN32
 #include <Mstcpip.h>
 #include <process.h>
@@ -33,8 +33,7 @@
 #define MONGOC_LOG_DOMAIN "socket"
 
 
-#define OPERATION_EXPIRED(expire_at) \
-   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
+#define OPERATION_EXPIRED(expire_at) ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
 
 
 /* either struct sockaddr or void, depending on platform */
@@ -72,9 +71,11 @@ _mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
 /*
  *--------------------------------------------------------------------------
  *
- * _mongoc_socket_setnonblock --
+ * _mongoc_socket_setflags --
+ *
+ *       A helper to set socket flags. Sets to nonblocking mode. On
+ *       POSIX sets closeonexec.
  *
- *       A helper to set a socket in nonblocking mode.
  *
  * Returns:
  *       true if successful; otherwise false.
@@ -87,9 +88,9 @@ _mongoc_socket_capture_errno (mongoc_socket_t *sock) /* IN */
 
 static bool
 #ifdef _WIN32
-_mongoc_socket_setnonblock (SOCKET sd)
+_mongoc_socket_setflags (SOCKET sd)
 #else
-_mongoc_socket_setnonblock (int sd)
+_mongoc_socket_setflags (int sd)
 #endif
 {
 #ifdef _WIN32
@@ -98,8 +99,19 @@ _mongoc_socket_setnonblock (int sd)
 #else
    int flags;
 
-   flags = fcntl (sd, F_GETFL, sd);
-   return (-1 != fcntl (sd, F_SETFL, (flags | O_NONBLOCK)));
+   flags = fcntl (sd, F_GETFL);
+
+   if (-1 == fcntl (sd, F_SETFL, (flags | O_NONBLOCK))) {
+      return false;
+   }
+
+#ifdef FD_CLOEXEC
+   flags = fcntl (sd, F_GETFD);
+   if (-1 == fcntl (sd, F_SETFD, (flags | FD_CLOEXEC))) {
+      return false;
+   }
+#endif
+   return true;
 #endif
 }
 
@@ -190,8 +202,7 @@ _mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
       } else {
          timeout_tv.tv_sec = timeout / 1000;
          timeout_tv.tv_usec = (timeout % 1000) * 1000;
-         ret = select (
-            0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+         ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
       }
       if (ret == SOCKET_ERROR) {
          _mongoc_socket_capture_errno (sock);
@@ -207,8 +218,7 @@ _mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
       if (ret > 0) {
 /* Something happened, so return that */
 #ifdef _WIN32
-         return (FD_ISSET (sock->sd, &read_fds) ||
-                 FD_ISSET (sock->sd, &write_fds));
+         return (FD_ISSET (sock->sd, &read_fds) || FD_ISSET (sock->sd, &write_fds));
 #else
          RETURN (0 != (pfd.revents & events));
 #endif
@@ -279,7 +289,6 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
    struct pollfd *pfds;
 #endif
    int ret;
-   int i;
 
    ENTRY;
 
@@ -290,7 +299,7 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
    FD_ZERO (&write_fds);
    FD_ZERO (&error_fds);
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       if (sds[i].events & POLLIN) {
          FD_SET (sds[i].socket->sd, &read_fds);
       }
@@ -312,7 +321,7 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
       return -1;
    }
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       if (FD_ISSET (sds[i].socket->sd, &read_fds)) {
          sds[i].revents = POLLIN;
       } else if (FD_ISSET (sds[i].socket->sd, &write_fds)) {
@@ -326,14 +335,14 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
 #else
    pfds = (struct pollfd *) bson_malloc (sizeof (*pfds) * nsds);
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       pfds[i].fd = sds[i].socket->sd;
       pfds[i].events = sds[i].events | POLLERR | POLLHUP;
       pfds[i].revents = 0;
    }
 
    ret = poll (pfds, nsds, timeout);
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       sds[i].revents = pfds[i].revents;
    }
 
@@ -346,7 +355,7 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
 
 /* https://jira.mongodb.org/browse/CDRIVER-2176 */
 #define MONGODB_KEEPALIVEINTVL 10
-#define MONGODB_KEEPIDLE 300
+#define MONGODB_KEEPIDLE 120
 #define MONGODB_KEEPALIVECNT 9
 
 #ifdef _WIN32
@@ -359,8 +368,7 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
    DWORD type;
    DWORD data;
    DWORD data_size = sizeof data;
-   const char *reg_key =
-      "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
+   const char *reg_key = "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
    keepalive.onoff = true;
    keepalive.keepalivetime = MONGODB_KEEPIDLE * 1000;
    keepalive.keepaliveinterval = MONGODB_KEEPALIVEINTVL * 1000;
@@ -376,16 +384,13 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
     * change the default value by setting the registry values.
     */
 
-   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) ==
-       ERROR_SUCCESS) {
+   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS) {
       /* https://technet.microsoft.com/en-us/library/cc957549.aspx */
       DWORD default_keepalivetime = 7200000; /* 2 hours */
       /* https://technet.microsoft.com/en-us/library/cc957548.aspx */
       DWORD default_keepaliveinterval = 1000; /* 1 second */
 
-      if (RegQueryValueEx (
-             hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) ==
-          ERROR_SUCCESS) {
+      if (RegQueryValueEx (hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) == ERROR_SUCCESS) {
          if (type == REG_DWORD && data < keepalive.keepalivetime) {
             keepalive.keepalivetime = data;
          }
@@ -393,12 +398,7 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
          keepalive.keepalivetime = default_keepalivetime;
       }
 
-      if (RegQueryValueEx (hKey,
-                           "KeepAliveInterval",
-                           NULL,
-                           &type,
-                           (LPBYTE) &data,
-                           &data_size) == ERROR_SUCCESS) {
+      if (RegQueryValueEx (hKey, "KeepAliveInterval", NULL, &type, (LPBYTE) &data, &data_size) == ERROR_SUCCESS) {
          if (type == REG_DWORD && data < keepalive.keepaliveinterval) {
             keepalive.keepaliveinterval = data;
          }
@@ -407,24 +407,17 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
       }
       RegCloseKey (hKey);
    }
-   if (WSAIoctl (sd,
-                 SIO_KEEPALIVE_VALS,
-                 &keepalive,
-                 sizeof keepalive,
-                 NULL,
-                 0,
-                 &lpcbBytesReturned,
-                 NULL,
-                 NULL) == SOCKET_ERROR) {
+   if (WSAIoctl (sd, SIO_KEEPALIVE_VALS, &keepalive, sizeof keepalive, NULL, 0, &lpcbBytesReturned, NULL, NULL) ==
+       SOCKET_ERROR) {
       TRACE ("%s", "Could not set keepalive values");
    } else {
       TRACE ("%s", "KeepAlive values updated");
-      TRACE ("KeepAliveTime: %d", keepalive.keepalivetime);
-      TRACE ("KeepAliveInterval: %d", keepalive.keepaliveinterval);
+      TRACE ("KeepAliveTime: %lu", keepalive.keepalivetime);
+      TRACE ("KeepAliveInterval: %lu", keepalive.keepaliveinterval);
    }
 }
 #else
-#ifdef MONGOC_TRACE
+
 static const char *
 _mongoc_socket_sockopt_value_to_name (int value)
 {
@@ -450,7 +443,7 @@ _mongoc_socket_sockopt_value_to_name (int value)
       return "Unknown option name";
    }
 }
-#endif
+
 static void
 _mongoc_socket_set_sockopt_if_less (int sd, int name, int value)
 {
@@ -459,25 +452,15 @@ _mongoc_socket_set_sockopt_if_less (int sd, int name, int value)
 
    optlen = sizeof optval;
    if (getsockopt (sd, IPPROTO_TCP, name, (char *) &optval, &optlen)) {
-      TRACE ("Getting '%s' failed, errno: %d",
-             _mongoc_socket_sockopt_value_to_name (name),
-             errno);
+      TRACE ("Getting '%s' failed, errno: %d", _mongoc_socket_sockopt_value_to_name (name), errno);
    } else {
-      TRACE ("'%s' is %d, target value is %d",
-             _mongoc_socket_sockopt_value_to_name (name),
-             optval,
-             value);
+      TRACE ("'%s' is %d, target value is %d", _mongoc_socket_sockopt_value_to_name (name), optval, value);
       if (optval > value) {
          optval = value;
-         if (setsockopt (
-                sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
-            TRACE ("Setting '%s' failed, errno: %d",
-                   _mongoc_socket_sockopt_value_to_name (name),
-                   errno);
+         if (setsockopt (sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
+            TRACE ("Setting '%s' failed, errno: %d", _mongoc_socket_sockopt_value_to_name (name), errno);
          } else {
-            TRACE ("'%s' value changed to %d",
-                   _mongoc_socket_sockopt_value_to_name (name),
-                   optval);
+            TRACE ("'%s' value changed to %d", _mongoc_socket_sockopt_value_to_name (name), optval);
          }
       }
    }
@@ -495,8 +478,7 @@ _mongoc_socket_setkeepalive_nix (int sd)
 #endif
 
 #ifdef TCP_KEEPINTVL
-   _mongoc_socket_set_sockopt_if_less (
-      sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
 #else
    TRACE ("%s", "TCP_KEEPINTVL not available");
 #endif
@@ -521,8 +503,7 @@ _mongoc_socket_setkeepalive (int sd) /* IN */
 
    ENTRY;
 #ifdef SO_KEEPALIVE
-   if (!setsockopt (
-          sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
+   if (!setsockopt (sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
       TRACE ("%s", "Setting SO_KEEPALIVE");
 #ifdef _WIN32
       _mongoc_socket_setkeepalive_windows (sd);
@@ -557,8 +538,7 @@ _mongoc_socket_setnodelay (int sd) /* IN */
    ENTRY;
 
    errno = 0;
-   ret = setsockopt (
-      sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
+   ret = setsockopt (sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
 
 #ifdef _WIN32
    if (ret == SOCKET_ERROR) {
@@ -702,7 +682,7 @@ again:
       RETURN (NULL);
    } else if (failed) {
       RETURN (NULL);
-   } else if (!_mongoc_socket_setnonblock (sd)) {
+   } else if (!_mongoc_socket_setflags (sd)) {
 #ifdef _WIN32
       closesocket (sd);
 #else
@@ -866,8 +846,7 @@ mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
    if (failed && try_again) {
       if (_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
          optval = -1;
-         ret = getsockopt (
-            sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
+         ret = getsockopt (sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
          if ((ret == 0) && (optval == 0)) {
             RETURN (0);
          } else {
@@ -980,6 +959,9 @@ mongoc_socket_new (int domain,   /* IN */
 #else
    int sd;
 #endif
+#ifdef SO_NOSIGPIPE
+   int on = 1;
+#endif
 
    ENTRY;
 
@@ -993,7 +975,7 @@ mongoc_socket_new (int domain,   /* IN */
       RETURN (NULL);
    }
 
-   if (!_mongoc_socket_setnonblock (sd)) {
+   if (!_mongoc_socket_setflags (sd)) {
       GOTO (fail);
    }
 
@@ -1004,6 +986,12 @@ mongoc_socket_new (int domain,   /* IN */
       _mongoc_socket_setkeepalive (sd);
    }
 
+   /* Set SO_NOSIGPIPE, to ignore SIGPIPE on writes for platforms where
+      setting MSG_NOSIGNAL on writes is not supported (primarily OSX). */
+#ifdef SO_NOSIGPIPE
+   setsockopt (sd, SOL_SOCKET, SO_NOSIGPIPE, &on, sizeof (on));
+#endif
+
    sock = (mongoc_socket_t *) bson_malloc0 (sizeof *sock);
    sock->sd = sd;
    sock->domain = domain;
@@ -1076,8 +1064,7 @@ again:
 #endif
    if (failed) {
       _mongoc_socket_capture_errno (sock);
-      if (_mongoc_socket_errno_is_again (sock) &&
-          _mongoc_socket_wait (sock, POLLIN, expire_at)) {
+      if (_mongoc_socket_errno_is_again (sock) && _mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
    }
@@ -1193,8 +1180,6 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
                                size_t iovcnt)         /* IN */
 {
    ssize_t ret = 0;
-   size_t i;
-   ssize_t wrote;
 
    ENTRY;
 
@@ -1202,11 +1187,13 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
    BSON_ASSERT (iov);
    BSON_ASSERT (iovcnt);
 
-   for (i = 0; i < iovcnt; i++) {
-      wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
+   for (size_t i = 0u; i < iovcnt; i++) {
 #ifdef _WIN32
+      BSON_ASSERT (bson_in_range_unsigned (int, iov[i].iov_len));
+      const int wrote = send (sock->sd, iov[i].iov_base, (int) iov[i].iov_len, 0);
       if (wrote == SOCKET_ERROR) {
 #else
+      const ssize_t wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
       if (wrote == -1) {
 #endif
          _mongoc_socket_capture_errno (sock);
@@ -1219,7 +1206,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
 
       ret += wrote;
 
-      if (wrote != iov[i].iov_len) {
+      if (bson_cmp_not_equal_su (wrote, iov[i].iov_len)) {
          RETURN (ret);
       }
    }
@@ -1269,16 +1256,13 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
    DUMP_IOVEC (sendbuf, iov, iovcnt);
 
 #ifdef _WIN32
-   ret = WSASend (
-      sock->sd, (LPWSABUF) iov, iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
-   TRACE ("WSASend sent: %ld (out of: %ld), ret: %d",
-          dwNumberofBytesSent,
-          iov->iov_len,
-          ret);
+   BSON_ASSERT (bson_in_range_unsigned (unsigned_long, iovcnt));
+   ret = WSASend (sock->sd, (LPWSABUF) iov, (DWORD) iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
+   TRACE ("WSASend sent: %ld (out of: %zu), ret: %d", dwNumberofBytesSent, iov->iov_len, ret);
 #else
    memset (&msg, 0, sizeof msg);
    msg.msg_iov = iov;
-   msg.msg_iovlen = (int) iovcnt;
+   msg.msg_iovlen = iovcnt;
    ret = sendmsg (sock->sd,
                   &msg,
 #ifdef MSG_NOSIGNAL
@@ -1286,7 +1270,7 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
 #else
                   0);
 #endif
-   TRACE ("Send %ld out of %ld bytes", ret, iov->iov_len);
+   TRACE ("Send %zu out of %zu bytes", ret, iov->iov_len);
 #endif
 
 
@@ -1366,8 +1350,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
 
    for (;;) {
       sent = _mongoc_socket_try_sendv (sock, &iov[cur], iovcnt - cur);
-      TRACE (
-         "Sent %ld (of %ld) out of iovcnt=%ld", sent, iov[cur].iov_len, iovcnt);
+      TRACE ("Sent %zd (of %zu) out of iovcnt=%zu", sent, iov[cur].iov_len, iovcnt);
 
       /*
        * If we failed with anything other than EAGAIN or EWOULDBLOCK,
@@ -1392,9 +1375,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * Subtract the sent amount from what we still need to send.
           */
          while ((cur < iovcnt) && (sent >= (ssize_t) iov[cur].iov_len)) {
-            TRACE ("still got bytes left: sent -= iov_len: %ld -= %ld",
-                   sent,
-                   iov[cur].iov_len);
+            TRACE ("still got bytes left: sent -= iov_len: %zd -= %zu", sent, iov[cur].iov_len);
             sent -= iov[cur++].iov_len;
          }
 
@@ -1411,12 +1392,11 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * Increment the current iovec buffer to its proper offset and adjust
           * the number of bytes to write.
           */
-         TRACE ("Seeked io_base+%ld", sent);
-         TRACE (
-            "Subtracting iov_len -= sent; %ld -= %ld", iov[cur].iov_len, sent);
+         TRACE ("Seeked io_base+%zd", sent);
+         TRACE ("Subtracting iov_len -= sent; %zu -= %zd", iov[cur].iov_len, sent);
          iov[cur].iov_base = ((char *) iov[cur].iov_base) + sent;
          iov[cur].iov_len -= sent;
-         TRACE ("iov_len remaining %ld", iov[cur].iov_len);
+         TRACE ("iov_len remaining %zu", iov[cur].iov_len);
 
          BSON_ASSERT (iovcnt - cur);
          BSON_ASSERT (iov[cur].iov_len);
@@ -1478,8 +1458,7 @@ mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
       RETURN (NULL);
    }
 
-   if (getnameinfo (
-          (struct sockaddr *) &addr, len, host, sizeof host, NULL, 0, 0)) {
+   if (getnameinfo ((struct sockaddr *) &addr, len, host, sizeof host, NULL, 0, 0)) {
       RETURN (NULL);
    }
 
@@ -1537,20 +1516,27 @@ mongoc_socket_inet_ntop (struct addrinfo *rp, /* IN */
 {
    void *ptr;
    char tmp[256];
+   int req;
 
    switch (rp->ai_family) {
    case AF_INET:
       ptr = &((struct sockaddr_in *) rp->ai_addr)->sin_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
-      bson_snprintf (buf, buflen, "ipv4 %s", tmp);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "ipv4 %s", tmp);
+      BSON_ASSERT (req > 0);
       break;
    case AF_INET6:
       ptr = &((struct sockaddr_in6 *) rp->ai_addr)->sin6_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
-      bson_snprintf (buf, buflen, "ipv6 %s", tmp);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "ipv6 %s", tmp);
+      BSON_ASSERT (req > 0);
       break;
    default:
-      bson_snprintf (buf, buflen, "unknown ip %d", rp->ai_family);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "unknown ip %d", rp->ai_family);
+      BSON_ASSERT (req > 0);
       break;
    }
 }
