@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -194,7 +194,7 @@ _mongoc_topology_scanner_cb (
 }
 
 static void
-_server_session_init (mongoc_server_session_t *session, mongoc_topology_t *unused, bson_error_t *error)
+_server_session_init (void *session, void *unused, bson_error_t *error)
 {
    BSON_UNUSED (unused);
 
@@ -202,7 +202,7 @@ _server_session_init (mongoc_server_session_t *session, mongoc_topology_t *unuse
 }
 
 static void
-_server_session_destroy (mongoc_server_session_t *session, mongoc_topology_t *unused)
+_server_session_destroy (void *session, void *unused)
 {
    BSON_UNUSED (unused);
 
@@ -210,10 +210,13 @@ _server_session_destroy (mongoc_server_session_t *session, mongoc_topology_t *un
 }
 
 static int
-_server_session_should_prune (mongoc_server_session_t *session, mongoc_topology_t *topo)
+_server_session_should_prune (const void *session_vp, void *topo_vp)
 {
-   BSON_ASSERT_PARAM (session);
-   BSON_ASSERT_PARAM (topo);
+   BSON_ASSERT_PARAM (session_vp);
+   BSON_ASSERT_PARAM (topo_vp);
+
+   const mongoc_server_session_t *const session = session_vp;
+   mongoc_topology_t *const topo = topo_vp;
 
    /** If "dirty" (i.e. contains a network error), it should be dropped */
    if (session->dirty) {
@@ -282,7 +285,7 @@ _mongoc_apply_srv_max_hosts (const mongoc_host_list_t *hl, size_t max_hosts, siz
     * Drivers SHOULD use the `Fisher-Yates shuffle` for randomization. */
    for (size_t idx = hl_size - 1u; idx > 0u; --idx) {
       /* 0 <= swap_pos <= idx */
-      const size_t swap_pos = _mongoc_rand_size_t (0u, idx, _mongoc_simple_rand_size_t);
+      const size_t swap_pos = _mongoc_rand_size_t (0u, idx);
 
       const mongoc_host_list_t *tmp = hl_array[swap_pos];
       hl_array[swap_pos] = hl_array[idx];
@@ -480,7 +483,7 @@ mongoc_topology_new (const mongoc_uri_t *uri, bool single_threaded)
        * lookup fails, SRV polling will still start when background monitoring
        * starts. */
       topology->srv_polling_last_scan_ms = bson_get_monotonic_time () / 1000;
-      topology->srv_polling_rescan_interval_ms = MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS;
+      _mongoc_topology_set_srv_polling_rescan_interval_ms (topology, MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS);
 
       /* a mongodb+srv URI. try SRV lookup, if no error then also try TXT */
       prefixed_hostname = bson_strdup_printf ("_%s._tcp.%s", mongoc_uri_get_srv_service_name (uri), srv_hostname);
@@ -518,8 +521,8 @@ mongoc_topology_new (const mongoc_uri_t *uri, bool single_threaded)
 
       topology->srv_polling_last_scan_ms = bson_get_monotonic_time () / 1000;
       /* TODO (CDRIVER-4047) use BSON_MIN */
-      topology->srv_polling_rescan_interval_ms =
-         BSON_MAX (rr_data.min_ttl * 1000, MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS);
+      int64_t new_iv = BSON_MAX (rr_data.min_ttl * 1000, MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS);
+      _mongoc_topology_set_srv_polling_rescan_interval_ms (topology, new_iv);
 
       topology->valid = true;
    srv_fail:
@@ -640,15 +643,15 @@ mongoc_topology_new (const mongoc_uri_t *uri, bool single_threaded)
 void
 mongoc_topology_set_apm_callbacks (mongoc_topology_t *topology,
                                    mongoc_topology_description_t *td,
-                                   mongoc_apm_callbacks_t *callbacks,
+                                   mongoc_apm_callbacks_t const *callbacks,
                                    void *context)
 {
    if (callbacks) {
-      memcpy (&td->apm_callbacks, callbacks, sizeof (mongoc_apm_callbacks_t));
-      memcpy (&topology->scanner->apm_callbacks, callbacks, sizeof (mongoc_apm_callbacks_t));
+      td->apm_callbacks = *callbacks;
+      topology->scanner->apm_callbacks = *callbacks;
    } else {
-      memset (&td->apm_callbacks, 0, sizeof (mongoc_apm_callbacks_t));
-      memset (&topology->scanner->apm_callbacks, 0, sizeof (mongoc_apm_callbacks_t));
+      td->apm_callbacks = (mongoc_apm_callbacks_t){0};
+      topology->scanner->apm_callbacks = (mongoc_apm_callbacks_t){0};
    }
 
    td->apm_context = context;
@@ -816,7 +819,7 @@ mongoc_topology_rescan_srv (mongoc_topology_t *topology)
    BSON_ASSERT (mongoc_topology_should_rescan_srv (topology));
 
    srv_hostname = mongoc_uri_get_srv_hostname (topology->uri);
-   scan_time_ms = topology->srv_polling_last_scan_ms + topology->srv_polling_rescan_interval_ms;
+   scan_time_ms = topology->srv_polling_last_scan_ms + _mongoc_topology_get_srv_polling_rescan_interval_ms (topology);
    if (bson_get_monotonic_time () / 1000 < scan_time_ms) {
       /* Query SRV no more frequently than srv_polling_rescan_interval_ms. */
       return;
@@ -839,14 +842,14 @@ mongoc_topology_rescan_srv (mongoc_topology_t *topology)
    topology->srv_polling_last_scan_ms = bson_get_monotonic_time () / 1000;
    if (!ret) {
       /* Failed querying, soldier on and try again next time. */
-      topology->srv_polling_rescan_interval_ms = td.ptr->heartbeat_msec;
+      _mongoc_topology_set_srv_polling_rescan_interval_ms (topology, td.ptr->heartbeat_msec);
       MONGOC_ERROR ("SRV polling error: %s", topology->scanner->error.message);
       GOTO (done);
    }
 
    /* TODO (CDRIVER-4047) use BSON_MIN */
-   topology->srv_polling_rescan_interval_ms =
-      BSON_MAX (rr_data.min_ttl * 1000, MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS);
+   const int64_t new_iv = BSON_MAX (rr_data.min_ttl * 1000, MONGOC_TOPOLOGY_MIN_RESCAN_SRV_INTERVAL_MS);
+   _mongoc_topology_set_srv_polling_rescan_interval_ms (topology, new_iv);
 
    tdmod = mc_tpld_modify_begin (topology);
    if (!mongoc_topology_apply_scanned_srv_hosts (
@@ -861,7 +864,7 @@ mongoc_topology_rescan_srv (mongoc_topology_t *topology)
        * to heartbeatFrequencyMS until at least one verified SRV record is
        * obtained."
        */
-      topology->srv_polling_rescan_interval_ms = td.ptr->heartbeat_msec;
+      _mongoc_topology_set_srv_polling_rescan_interval_ms (topology, td.ptr->heartbeat_msec);
    }
    mc_tpld_modify_commit (tdmod);
 
@@ -1118,6 +1121,9 @@ mongoc_topology_select_server_id (mongoc_topology_t *topology,
    uint32_t server_id;
    mc_shared_tpld td = mc_tpld_take_ref (topology);
 
+   bson_string_t *topology_type = bson_string_new (". Topology type: ");
+   bson_string_append (topology_type, mongoc_topology_description_type (td.ptr));
+
    /* These names come from the Server Selection Spec pseudocode */
    int64_t loop_start;  /* when we entered this function */
    int64_t loop_end;    /* when we last completed a loop (single-threaded) */
@@ -1173,10 +1179,7 @@ mongoc_topology_select_server_id (mongoc_topology_t *topology,
 
             if (scan_ready > expire_at && !try_once) {
                /* selection timeout will expire before min heartbeat passes */
-               _mongoc_server_selection_error ("No suitable servers found: "
-                                               "`serverselectiontimeoutms` timed out",
-                                               &scanner_error,
-                                               error);
+               _mongoc_server_selection_error (timeout_msg, &scanner_error, error);
 
                server_id = 0;
                goto done;
@@ -1321,6 +1324,13 @@ mongoc_topology_select_server_id (mongoc_topology_t *topology,
    }
 
 done:
+   if (error && server_id == 0) {
+      /* server_id set to zero indicates that an error has occured and that `error` is initialized */
+      if (error->domain == MONGOC_ERROR_SERVER_SELECTION) {
+         _mongoc_error_append (error, topology_type->str);
+      }
+   }
+   bson_string_free (topology_type, true);
    mc_tpld_drop_ref (&td);
    return server_id;
 }
@@ -1889,12 +1899,6 @@ _mongoc_topology_set_rr_resolver (mongoc_topology_t *topology, _mongoc_rr_resolv
    topology->rr_resolver = rr_resolver;
 }
 
-void
-_mongoc_topology_set_srv_polling_rescan_interval_ms (mongoc_topology_t *topology, int64_t val)
-{
-   topology->srv_polling_rescan_interval_ms = val;
-}
-
 uint32_t
 _mongoc_topology_get_connection_pool_generation (const mongoc_topology_description_t *td,
                                                  uint32_t server_id,
